import { IDSelector, EntityAdapter } from './type';
import { createInitialStateFactory } from './state';
import { createSelectorsFactory } from './selectors';
import { createUnsortedStateAdapter } from './unsorted_state_adapter';

export function createEntityAdapter<T, E>(selectId?: IDSelector<T>): EntityAdapter<T, E>;
export function createEntityAdapter<T, E>(selectId: IDSelector<T>): EntityAdapter<T, E> {
  if (undefined === selectId) {
    selectId = (e: any) => e.id;
  }

  const stateFactory = createInitialStateFactory<T, E>();
  const selectorsFactory = createSelectorsFactory<T, E>();
  const stateAdapter = createUnsortedStateAdapter<T, E>(selectId);

  return {
    selectId,
    ...stateFactory,
    ...selectorsFactory,
    ...stateAdapter,
  };
}
