import { EntityState, EntitySelectors, Dictionary } from './type';
import { createSelector } from '@ngrx/store';


export function createSelectorsFactory<T, E>() {
  function getSelectors(): EntitySelectors<T, E, EntityState<T, E>>;
  function getSelectors<S>(selectState: (state: S) => EntityState<T, E>): EntitySelectors<T, E, S>;
  function getSelectors(selectState?: (state: any) => EntityState<T, E>): EntitySelectors<T, E, any> {
    const selectIds = (state: any) => state.ids;
    const selectEntities = (state: EntityState<T, E>) => state.entities;
    const selectAll = createSelector(
      selectIds,
      selectEntities,
      (ids: T[], entities: Dictionary<T>): any => ids.map((id: any) => (entities as any)[id])
    );
    const selectTotal = createSelector(selectIds, ids => ids.length);
    const selectError = (state: any) => state.error;

    if (!selectState) {
      return {
        selectIds,
        selectEntities,
        selectOne: (id: string | number) => createSelector(
          selectEntities,
          (entities: Dictionary<T>): T => {
            if (null !== entities) {
              return (entities as any)[id];
            }
            return null;
          }
        ),
        selectAll,
        selectTotal,
        selectError,
      };
    }

    return {
      selectIds: createSelector(selectState, selectIds),
      selectEntities: createSelector(selectState, selectEntities),
      selectOne: (id: string | number) => createSelector(
        selectState,
        createSelector(
          selectEntities,
          (entities: Dictionary<T>): T => {
            if (null !== entities) {
              return (entities as any)[id];
            }
            return null;
          }
        )
      ),
      selectAll: createSelector(selectState, selectAll),
      selectTotal: createSelector(selectState, selectTotal),
      selectError: createSelector(selectState, selectError),
    };
  }

  return {
    getSelectors
  };
}
