import { EntityState } from './type';

export enum DidMutate {
  EntitiesOnly,
  Both,
  Error,
  None,
}

export function createStateOperator<T, E, R>(mutator: (arg: R, state: EntityState<T, E>) => DidMutate): EntityState<T, E>;
export function createStateOperator<T, E, R>(mutator: (arg: any, state: any) => DidMutate): any {
  return function operation<S extends EntityState<T, E>>(arg: R, state: any): S {
    const clonedEntityState: EntityState<T, E> = {
      ids: [...state.ids],
      entities: { ...state.entities },
      error: state.error,
    };

    const didMutate = mutator(arg, clonedEntityState);

    if (didMutate === DidMutate.Both) {
      clonedEntityState.error = null;
      return Object.assign({}, state, clonedEntityState);
    }

    if (didMutate === DidMutate.EntitiesOnly) {
      return {
        ...state,
        entities: clonedEntityState.entities,
        error: null,
      };
    }

    if (didMutate === DidMutate.Error) {
      return {
        ...state,
        error: clonedEntityState.error,
      };
    }

    return state;
  };
}
