import {
  NgModule,
  ModuleWithProviders,
  Type,
  Inject,
} from '@angular/core';

import {
  _ROOT_SUBSCRIBERS,
  _FEATURE_SUBSCRIBERS,
} from './core/token';

import {
  RPCService,
} from './service/rpc.service';

import {
  RPCSubscribeService,
} from './subscribe/subscribe.service';

const SERVICES = [
  RPCService,
];

const SUBSCRIBERS = [
  RPCSubscribeService,
];

@NgModule({})
export class RPCRootModule {
  constructor(
    private rpcService: RPCService,
    private rpcSubscribeService: RPCSubscribeService,
    @Inject(_ROOT_SUBSCRIBERS) rootSubscribers: any[],
  ) {
    rootSubscribers.forEach((subscriber) => {
      rpcSubscribeService.addSubscriber(subscriber);
    });
  }
}

@NgModule({})
export class RPCFeatureModule {
  constructor(
    private rpcService: RPCService,
    private rpcSubscribeService: RPCSubscribeService,
    @Inject(_FEATURE_SUBSCRIBERS) featureSubscribersGroups: any[][],
    private root: RPCRootModule,
  ) {
    featureSubscribersGroups.forEach((featureSubscribers) => {
      featureSubscribers.forEach((subscriber) => {
        rpcSubscribeService.addSubscriber(subscriber);
      });
    });
  }
}

@NgModule({})
export class RPCModule {
  static forRoot(subscribers: Type<any>[]): ModuleWithProviders {
    return {
      ngModule: RPCRootModule,
      providers: [
        subscribers,
        {
          provide: _ROOT_SUBSCRIBERS,
          deps: subscribers,
          useFactory: createSourceInstances,
        },
        SERVICES,
        SUBSCRIBERS,
      ],
    };
  }

  static forFeature(subscribers: Type<any>[]): ModuleWithProviders {
    return {
      ngModule: RPCFeatureModule,
      providers: [
        subscribers,
        {
          provide: _FEATURE_SUBSCRIBERS,
          multi: true,
          deps: subscribers,
          useFactory: createSourceInstances,
        },
      ],
    };
  }
}

export function createSourceInstances(...instances: any[]) {
  return instances;
}
